package dev.shadowsoffire.apotheosis.adventure.affix;

package .var.folders._j.vfbv187d0bb8b8w5spzgg8180000gn.T;

import com.mojang.datafixers.kinds.App;
import com.mojang.datafixers.kinds.Applicative;
import com.mojang.datafixers.util.Function4;
import com.mojang.serialization.Codec;
import com.mojang.serialization.codecs.RecordCodecBuilder;
import dev.shadowsoffire.apotheosis.adventure.AdventureModule;
import dev.shadowsoffire.apotheosis.adventure.affix.Affix;
import dev.shadowsoffire.apotheosis.adventure.affix.AffixType;
import dev.shadowsoffire.apotheosis.adventure.loot.LootCategory;
import dev.shadowsoffire.apotheosis.adventure.loot.LootRarity;
import dev.shadowsoffire.apotheosis.adventure.socket.gem.bonus.GemBonus;
import dev.shadowsoffire.attributeslib.AttributesLib;
import dev.shadowsoffire.attributeslib.api.IFormattableAttribute;
import dev.shadowsoffire.placebo.codec.PlaceboCodecs;
import dev.shadowsoffire.placebo.util.StepFunction;
import java.util.Map;
import java.util.Set;
import java.util.function.BiConsumer;
import java.util.stream.Collectors;
import net.minecraft.ChatFormatting;
import net.minecraft.network.chat.Component;
import net.minecraft.network.chat.MutableComponent;
import net.minecraft.world.entity.EquipmentSlot;
import net.minecraft.world.entity.ai.attributes.Attribute;
import net.minecraft.world.entity.ai.attributes.AttributeModifier;
import net.minecraft.world.item.ItemStack;
import net.minecraftforge.registries.ForgeRegistries;
import org.apache.commons.lang3.tuple.Pair;

public class AttributeAffix extends Affix {
  public static final Codec<dev.shadowsoffire.apotheosis.adventure.affix.AttributeAffix> CODEC;
  
  protected final Attribute attribute;
  
  protected final AttributeModifier.Operation operation;
  
  protected final Map<LootRarity, StepFunction> values;
  
  protected final Set<LootCategory> types;
  
  protected final transient Map<LootRarity, ModifierInst> modifiers;
  
  static {
    CODEC = RecordCodecBuilder.create(inst -> inst.group((App)ForgeRegistries.ATTRIBUTES.getCodec().fieldOf("attribute").forGetter(()), (App)PlaceboCodecs.enumCodec(AttributeModifier.Operation.class).fieldOf("operation").forGetter(()), (App)GemBonus.VALUES_CODEC.fieldOf("values").forGetter(()), (App)LootCategory.SET_CODEC.fieldOf("types").forGetter(())).apply((Applicative)inst, dev.shadowsoffire.apotheosis.adventure.affix.AttributeAffix::new));
  }
  
  public AttributeAffix(Attribute attr, AttributeModifier.Operation op, Map<LootRarity, StepFunction> values, Set<LootCategory> types) {
    super(AffixType.STAT);
    this.attribute = attr;
    this.operation = op;
    this.values = values;
    this.types = types;
    this.modifiers = (Map<LootRarity, ModifierInst>)values.entrySet().stream().map(entry -> Pair.of(entry.getKey(), new ModifierInst(attr, op, (StepFunction)entry.getValue()))).collect(Collectors.toMap(Pair::getKey, Pair::getValue));
  }
  
  public MutableComponent getDescription(ItemStack stack, LootRarity rarity, float level) {
    return Component.m_237119_();
  }
  
  public Component getAugmentingText(ItemStack stack, LootRarity rarity, float level) {
    MutableComponent comp;
    ModifierInst modif = this.modifiers.get(rarity);
    double value = modif.valueFactory.get(level);
    MutableComponent valueComp = IFormattableAttribute.toValueComponent(this.attribute, this.operation, (value < 0.0D) ? -value : value, AttributesLib.getTooltipFlag());
    if (value > 0.0D) {
      comp = Component.m_237110_("attributeslib.modifier.plus", new Object[] { valueComp, Component.m_237115_(this.attribute.m_22087_()) }).m_130940_(ChatFormatting.BLUE);
    } else {
      comp = Component.m_237110_("attributeslib.modifier.take", new Object[] { valueComp, Component.m_237115_(this.attribute.m_22087_()) }).m_130940_(ChatFormatting.RED);
    } 
    if (modif.valueFactory.get(0.0F) != modif.valueFactory.get(1.0F)) {
      MutableComponent mutableComponent1 = IFormattableAttribute.toValueComponent(this.attribute, this.operation, modif.valueFactory.get(0.0F), AttributesLib.getTooltipFlag());
      MutableComponent mutableComponent2 = IFormattableAttribute.toValueComponent(this.attribute, this.operation, modif.valueFactory.get(1.0F), AttributesLib.getTooltipFlag());
      comp.m_7220_((Component)valueBounds((Component)mutableComponent1, (Component)mutableComponent2));
    } 
    return (Component)comp;
  }
  
  public void addModifiers(ItemStack stack, LootRarity rarity, float level, EquipmentSlot type, BiConsumer<Attribute, AttributeModifier> map) {
    LootCategory cat = LootCategory.forItem(stack);
    if (cat.isNone()) {
      AdventureModule.LOGGER.debug("Attempted to apply the attributes of affix {} on item {}, but it is not an affix-compatible item!", getId(), stack.m_41786_().getString());
      return;
    } 
    ModifierInst modif = this.modifiers.get(rarity);
    if (modif.attr == null) {
      AdventureModule.LOGGER.debug("The affix {} has attempted to apply a null attribute modifier to {}!", getId(), stack.m_41786_().getString());
      return;
    } 
    for (EquipmentSlot slot : cat.getSlots()) {
      if (slot == type)
        map.accept(modif.attr, modif.build(stack, getId(), level)); 
    } 
  }
  
  public boolean canApplyTo(ItemStack stack, LootCategory cat, LootRarity rarity) {
    if (cat.isNone())
      return false; 
    return ((this.types.isEmpty() || this.types.contains(cat)) && this.modifiers.containsKey(rarity));
  }
  
  public Codec<? extends Affix> getCodec() {
    return (Codec)CODEC;
  }
}
